from Bio import SeqIO
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC
from Bio.SeqRecord import SeqRecord


# put sequence in single line
# SeqIO.write(seq_record, output_handle, 'fasta-2line')

'''
Turn a SeqRecord into a two-line FASTA formated string.
This is used internally by the SeqRecord’s .format(“fasta-2line”) method and by the SeqIO.write(…, …, “fasta-2line”) function.
'''


def export_dna_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.unambiguous_dna)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


gene_seq = 'AAAAAAAAAAAAAAAAAAA'
gene_id = 'Hsp'
gene_description = ''
pwd_output_file = '/Users/songweizhi/Desktop/test_plot_flankings/AAM_00209___BNM_00032/test.fasta'


output_handle = open(pwd_output_file, 'w')
export_dna_record(gene_seq, gene_id, gene_description, output_handle)
output_handle.close()




from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqIO import FastaIO
from Bio.Alphabet import IUPAC
from Bio.SeqRecord import SeqRecord


def export_dna_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.unambiguous_dna)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    fasta_out = FastaIO.FastaWriter(output_handle, wrap=None)
    fasta_out.write_header()
    fasta_out.write_record(seq_record)
    fasta_out.write_footer()
    
    

for record in SeqIO.parse('gbk_file', "genbank"):
    taxonomy_list = record.annotations['taxonomy']
    taxonomy_str = ','.join(taxonomy_list)

    
    
    


import copy
from Bio import Phylo


def get_child_nodes(tree, node_id):
    for clade in tree.find_clades():
        if str(clade.name) == node_id:
            all_children_nodes = clade.get_terminals() + clade.get_nonterminals()

            all_children_nodes_self_excluded = []
            for nonleaf_node in all_children_nodes:
                if str(nonleaf_node.name) != node_id:
                    all_children_nodes_self_excluded.append(str(nonleaf_node.name))

            return all_children_nodes_self_excluded


# not perfect
def subset_tree(tree, taxon_to_keep):

    # add tmp value to None clade
    name_index = 0
    for clade in tree.find_clades():
        if clade.name == None:
            clade.name = 'TmpAdd_%s' % name_index
        name_index += 1

    for clade in tree.find_clades():
        if str(clade.name) in taxon_to_keep:
            clade_child_nodes = get_child_nodes(tree, str(clade.name))
            for child_node in clade_child_nodes:
                tree.collapse(child_node)

    all_leaf_clades = tree.get_terminals()
    all_leaf_clades_id = [str(i.name) for i in all_leaf_clades]
    all_leaf_clades_id_to_collapse = []
    for each_leaf in all_leaf_clades_id:
        if each_leaf not in taxon_to_keep:
            all_leaf_clades_id_to_collapse.append(each_leaf)

    for leaf_to_collapse in all_leaf_clades_id_to_collapse:
        tree.collapse(leaf_to_collapse)

    return tree


######################################################### demo #########################################################

tree_file_in = '/Users/songweizhi/Desktop/tree/example.tree'
tree_in = Phylo.read(tree_file_in, 'newick')
tree_new = copy.deepcopy(tree_in)


#tree_new.get_terminals('X', order='postorder')
# , order='preorder' postorder level
#name_list = ['A']
#for name in name_list:
    #tree_new.prune(name)
    #tree_new.collapse(name)


#print(tree_new.collapse_all('X'))
#tree_new.collapse('A')
#tree_new.collapse('B')
#tree_new.collapse('I')
#tree_new.prune('I')

#print(tree_new.find_any('X'))





# for clade in tree_new.find_clades():
#     if str(clade.name) in taxon_to_keep:
#         clade_child_nodes = get_child_nodes(tree_new, str(clade.name))
#
#         print(clade_child_nodes)
#         # for child_node in clade_child_nodes:
#         #
#         #     print(child_node)
#         #     #tree.collapse(child_node)

#
# taxon_to_keep = ['H', 'I']
# tree_new = subset_tree(tree_new, taxon_to_keep)
# tree_new = subset_tree(tree_new, taxon_to_keep)
# tree_new = subset_tree(tree_new, taxon_to_keep)
# tree_new = subset_tree(tree_new, taxon_to_keep)
# tree_new = subset_tree(tree_new, taxon_to_keep)
#

# to compare
print('\n########## For comparison ##########\n')
print(tree_in)
print('\n')
print(tree_new)
print('\n')
Phylo.draw_ascii(tree_in)
print('\n')
Phylo.draw_ascii(tree_new)



import argparse


BestHit_parser_usage = '''
================ BestHit example commands ================

BioSAK BestHit -i blast_result.tab -o blast_result_BestHit.tab

===============================================================
'''


def best_hit(args):

    file_in = args['i']
    file_out = args['o']

    file_out_handle = open(file_out, 'w')
    best_hit_line = ''
    best_hit_query_id = ''
    best_hit_score = 0
    for blast_hit in open(file_in):
        blast_hit_split = blast_hit.strip().split('\t')
        query_id = blast_hit_split[0]
        bit_score = float(blast_hit_split[11])

        if best_hit_query_id == '':
            best_hit_query_id = query_id
            best_hit_line = blast_hit
            best_hit_score = bit_score

        elif (query_id == best_hit_query_id) and (bit_score > best_hit_score):
            best_hit_score = bit_score
            best_hit_line = blast_hit

        elif query_id != best_hit_query_id:
            file_out_handle.write(best_hit_line)
            best_hit_query_id = query_id
            best_hit_line = blast_hit
            best_hit_score = bit_score

    file_out_handle.write(best_hit_line)
    file_out_handle.close()


if __name__ == '__main__':

    BestHit_parser = argparse.ArgumentParser()

    # arguments for BestHit_parser
    BestHit_parser.add_argument('-i',                   required=True,                                  help='input blast results (outfmt: 6)')
    BestHit_parser.add_argument('-o',                   required=True,                                  help='output file')

    args = vars(BestHit_parser.parse_args())

    best_hit(args)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import requests
from requests.auth import HTTPBasicAuth


class BacdiveClient(object):
#     This class provides method to get taxonomy informations corresponding
#     users entry(speciesname)
#
#     The attributes of the class are:
#     * ``headers`` -- sets the content-type of the HTTP request-header to json
#     * ``credentials`` -- attaches the username and the password to HTTPBasicAuth for using with the `requests` library

    headers = {'Accept': 'application/json'}
    USERNAME = 'songwz03@gmail.com'
    PASSWORD = '52shan88A'
    credentials = HTTPBasicAuth(USERNAME, PASSWORD)

#   _____________________________________________________________________________________________________________________________________
    def getAllLinks(self):
#         To get a list with links to the Detail View for BacDive ID, but only for the first page.
#         To consider the other pages, you habve to change the url to 'http://bacdive.dsmz.de/api/bacdive/bacdive_id/?page=2' etc.
#
#         INPUT:
        response = requests.get(
            'http://bacdive.dsmz.de/api/bacdive/bacdive_id/',
            headers=self.headers,
            auth=self.credentials
        )

        if response.status_code == 200:
            results = response.json()

#             OUTPUT:
#             object of type 'dict' with the fields 'count', 'previous', 'results', 'next'
#             the entries in field 'results' contain the URLs to correspondong strains separated by ',' e.g. {u'url:url1},{u'url:url2},{u'url:url3},etc
            return results

#   _____________________________________________________________________________________________________________________________________
    def getLinkByCultureno(self,culturecolnumber):
#         To get an URL to a strain corresponding the culture collection number
#
#         INPUT:
        response = requests.get(
            'http://bacdive.dsmz.de/api/bacdive/culturecollectionno/%s/' % (culturecolnumber),
            headers=self.headers,
            auth=self.credentials
        )

        if response.status_code == 200:
            results = response.json()

#             OUTPUT :
#             object of type 'dict' with the URL correspondong the culture collection number  e.g. {u'url:url1}
            return results

#   _____________________________________________________________________________________________________________________________________
    def getLinksByGenus(self, genus):
#         To get a list with links to the Detail View for BacDive ID, but only for the first page.
#         To consider the other pages, you habve to change the url to 'http://bacdive.dsmz.de/api/bacdive/taxon/{genus}/?page=2' etc.
#
#         INPUT:
        response = requests.get(
            'http://bacdive.dsmz.de/api/bacdive/taxon/%s/' % (genus ),
            headers=self.headers,
            auth=self.credentials
        )

        if response.status_code == 200:
            results = response.json()

#             OUTPUT:
#             object of type 'dict' with the fields 'count', 'previous', 'results', 'next'
#             the entries in field 'results' contain the URLs to correspondong strains separated by ',' e.g. {u'url:url1},{u'url:url2},{u'url:url3},etc
            return results

#   _____________________________________________________________________________________________________________________________________
    def getLinksBySpecies(self, genus, species_epithet):
#         To get a list with links to the Detail View for BacDive ID, but only for the first page.
#         To consider the other pages, you habve to change the url to 'http://bacdive.dsmz.de/api/bacdive/taxon/{genus}/{species_epithet}/?page=2' etc.
#
#         INPUT:
        response = requests.get(
            'http://bacdive.dsmz.de/api/bacdive/taxon/%s/%s/' % (genus,species_epithet),
            headers=self.headers,
            auth=self.credentials
        )

        if response.status_code == 200:
            results = response.json()

#             OUTPUT:
#             object of type 'dict' with the fields 'count', 'previous', 'results', 'next'
#             the entries in field 'results' contain the URLs to correspondong strains separated by ',' e.g. {u'url:url1},{u'url:url2},{u'url:url3},etc
            return results

#   _____________________________________________________________________________________________________________________________________
    def getLinksBySubspecies(self, genus, species_epithet, subspecies_epithet):
#         To get a list with links to the Detail View for BacDive ID, but only for the first page.
#         To consider the other pages, you habve to change the url to 'http://bacdive.dsmz.de/api/bacdive/taxon/{genus}/{species_epithet}/{subspecies_epithet}/?page=2' etc.
#
#         INPUT:
        response = requests.get(
            'http://bacdive.dsmz.de/api/bacdive/taxon/%s/%s/%s/' % (genus,species_epithet,subspecies_epithet),
            headers=self.headers,
            auth=self.credentials
        )

        if response.status_code == 200:
            results = response.json()

#             OUTPUT:
#             object of type 'dict' with the fields 'count', 'previous', 'results', 'next'
#             the entries in field 'results' contain the URLs to correspondong strains separated by ',' e.g. {u'url:url1},{u'url:url2},{u'url:url3},etc
            return results

#   _____________________________________________________________________________________________________________________________________
    def getLinksBySeqAccNum(self, seq_acc_num):
#         To get a list with links to the Detail View for BacDive ID. Usually it is a single entry and rarely few, mostly 2 or 3.
#
#         INPUT:
        response = requests.get('http://bacdive.dsmz.de/api/bacdive/sequence/%s/' % (seq_acc_num),
            headers=self.headers,
            auth=self.credentials
        )

        if response.status_code == 200:
            results = response.json()
#             OUTPUT:
#             object of type 'dict' with the fields 'count', 'previous', 'results', 'next'
#             the entries in field 'results' contain the URLs to correspondong strains separated by ',' e.g. {u'url:url1},{u'url:url2},{u'url:url3},etc
            return results
#   _____________________________________________________________________________________________________________________________________
    def run(self):
#         Run the client
        allLinks = self.getAllLinks()
        print(allLinks)
        cultureno = self.getLinkByCultureno('DSM 1')
        print(cultureno)
        genus = self.getLinksByGenus('acetobacter')
        print(genus)
        species = self.getLinksBySpecies('acetobacter','aceti')
        print(species)
        subspecies = self.getLinksBySubspecies('Bacillus','subtilis', 'subtilis')
        print(subspecies)
        sec_acc_num = self.getLinksBySeqAccNum('ALAS01000001')
        print(sec_acc_num)

if __name__ == '__main__':
    BacdiveClient().run()



import os
import shutil
import argparse
import multiprocessing as mp


sra_reads_downloader_usage = '''
============================ sra_reads_downloader example commands =============================

module load sratoolkit/2.9.6-1
BioSAK sra_reads_downloader -in prokaryotes.csv

================================================================================================
'''


def dwnld_sra_reads_worker():

    pass


def dwnld_sra_reads(args):

    pass


args = {'i': 'SRR_Acc_List_1.txt',
        'o': 'test_out',
        't': 4}


# if __name__ == '__main__':
#
#     parser = argparse.ArgumentParser()
#
#     parser.add_argument('-in', required=True, help='input csv file')
#
#     args = vars(parser.parse_args())
#
#     sra_reads_downloader(args)
#



import io
import os
import argparse
import itertools
import pandas as pd
from ete3 import Tree
from string import ascii_uppercase
from scipy.cluster.hierarchy import linkage
from matplotlib import pyplot as plt
from scipy.cluster.hierarchy import dendrogram, linkage
import numpy as np


def get_node_distance(tree, node_1, node_2):
    distance = tree.get_distance(node_1, node_2)
    return distance


def get_matrix(input_newick_file, output_matrix_file):
    # read in tree
    tree_in = Tree(input_newick_file, format=3)

    # get sorted leaf node list
    leaf_node_list = []
    for leaf_node in tree_in:
        leaf_node_list.append(leaf_node.name)

    leaf_node_list = sorted(leaf_node_list)

    # write out distance matrix
    matrix_out_handle = open(output_matrix_file, 'w')
    matrix_out_handle.write(',%s\n' % ','.join(leaf_node_list))
    for each_node_1 in leaf_node_list:
        current_node_distance_list = []
        for each_node_2 in leaf_node_list:
            distance = 0
            if each_node_1 != each_node_2:
                distance = get_node_distance(tree_in, each_node_1, each_node_2)
                distance = float("{0:.5f}".format(distance))
            current_node_distance_list.append(str(distance))
        matrix_out_handle.write('%s,%s\n' % (each_node_1, ','.join(current_node_distance_list)))

    matrix_out_handle.close()

os.chdir('/Users/songweizhi/Desktop')

newick_tree_file = 'species_tree.newick'
matrix_file = 'species_tree_with_group_matrix.csv'

# get distance matrix from tree
get_matrix(newick_tree_file, matrix_file)

# read in distance matrix as data frame
distance_matrix = pd.read_csv(matrix_file, index_col=0)

# read in distance matrix as numpy array
# import numpy as np
# distance_array = np.genfromtxt(matrix_file, delimiter=',', skip_header=True)
#
# print(distance_array.shape)
# # run clustering
#

#
# from scipy.cluster.hierarchy import dendrogram, linkage
# from matplotlib import pyplot as plt
#
#
# np.random.seed(4711)  # for repeatability of this tutorial
# a = np.random.multivariate_normal([10, 0], [[3, 1], [1, 4]], size=[100,])
# b = np.random.multivariate_normal([0, 20], [[3, 1], [1, 4]], size=[50,])
# X = np.concatenate((a, b),)
#
#
# print(a)
#
# print(X.shape)
# print(type(X))
#
# print(distance_matrix.shape)
# print(type(distance_matrix))



# Z = linkage(distance_matrix)
# Z
# dendrogram(Z)
#
# plt.show()


#
# def kMedoids(D, k, tmax=100):
#     # determine dimensions of distance matrix D
#     m, n = D.shape
#
#     if k > n:
#         raise Exception('too many medoids')
#
#     # find a set of valid initial cluster medoid indices since we
#     # can't seed different clusters with two points at the same location
#     valid_medoid_inds = set(range(n))
#     invalid_medoid_inds = set([])
#     rs,cs = np.where(D==0)
#     # the rows, cols must be shuffled because we will keep the first duplicate below
#     index_shuf = range(len(rs))
#     np.random.shuffle(index_shuf)
#     rs = rs[index_shuf]
#     cs = cs[index_shuf]
#     for r,c in zip(rs,cs):
#         # if there are two points with a distance of 0...
#         # keep the first one for cluster init
#         if r < c and r not in invalid_medoid_inds:
#             invalid_medoid_inds.add(c)
#     valid_medoid_inds = list(valid_medoid_inds - invalid_medoid_inds)
#
#     if k > len(valid_medoid_inds):
#         raise Exception('too many medoids (after removing {} duplicate points)'.format(
#             len(invalid_medoid_inds)))
#
#     # randomly initialize an array of k medoid indices
#     M = np.array(valid_medoid_inds)
#     np.random.shuffle(M)
#     M = np.sort(M[:k])
#
#     # create a copy of the array of medoid indices
#     Mnew = np.copy(M)
#
#     # initialize a dictionary to represent clusters
#     C = {}
#     for t in xrange(tmax):
#         # determine clusters, i. e. arrays of data indices
#         J = np.argmin(D[:,M], axis=1)
#         for kappa in range(k):
#             C[kappa] = np.where(J==kappa)[0]
#         # update cluster medoids
#         for kappa in range(k):
#             J = np.mean(D[np.ix_(C[kappa],C[kappa])],axis=1)
#             j = np.argmin(J)
#             Mnew[kappa] = C[kappa][j]
#         np.sort(Mnew)
#         # check for convergence
#         if np.array_equal(M, Mnew):
#             break
#         M = np.copy(Mnew)
#     else:
#         # final update of cluster memberships
#         J = np.argmin(D[:,M], axis=1)
#         for kappa in range(k):
#             C[kappa] = np.where(J==kappa)[0]
#
#     # return results
#     return M, C
#
#
#
# M,C = kMedoids(distance_matrix, 10, tmax=100)
#
# print(M)
# print(C)



import plotly.plotly as py
import plotly.figure_factory as ff
from plotly.graph_objs import *


data = [['', 'Emma', 'Isabella', 'Ava', 'Olivia', 'Sophia', 'row-sum'],
        ['Emma', 16, 3, 28, 0, 18, 65],
        ['Isabella', 18, 0, 12, 5, 29, 64],
        ['Ava', 9, 11, 17, 27, 0, 64],
        ['Olivia', 19, 0, 31, 11, 12, 73],
        ['Sophia', 23, 17, 10, 0, 34, 84]]


table = ff.create_table(data, index=True)

print(table)


import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


def get_boxplot(num_lol, label_list, label_rotation, output_plot):

    num_lol_arrary = [np.array(i) for i in num_lol]
    fig = plt.figure(1, figsize=(9, 6))
    ax = fig.add_subplot(111)
    bp = ax.boxplot(num_lol_arrary)
    ax.set_xticklabels(label_list, rotation=label_rotation, fontsize=8)

    ## change the style of fliers and their fill
    for flier in bp['fliers']:
        flier.set(marker='+', color='black', alpha=0.7, markersize=3)

    # add dots, https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.plot.html
    plt.plot(1, 7, alpha=1, marker='^', markersize=10, markeredgewidth=0, color='coral')
    plt.plot(2, 2, alpha=1, marker='v', markersize=10, markeredgewidth=0, color='deepskyblue')
    plt.plot(3, 4, alpha=1, marker='v', markersize=10, markeredgewidth=0, color='deepskyblue')

    plt.tight_layout()
    fig.savefig(output_plot, bbox_inches='tight', dpi=300)
    plt.close()


num_lol = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
label_list = ['Apl', 'Car', 'Cli']
label_rotation = 0
output_plot = '/Users/songweizhi/Desktop/Boxplot_with_extra_values.png'
get_boxplot(num_lol, label_list, label_rotation, output_plot)





import argparse
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord


slice_seq_usage = '''
========================== slice_seq example commands ==========================

BioSAK slice_seq -in idba.fa -id ctg_01 -range 20-400 -out ctg1_20_400bp.fa
BioSAK slice_seq -in idba.fa -id ctg_01 -range 1-50 -out ctg1_1_50bp.fa -rc
BioSAK slice_seq -in idba.fa -id ctg_01 -range 30-end -out ctg1_30bp_to_end.fa

================================================================================
'''


def slice_seq(args):

    seq_file    = args['in']
    ctg_id      = args['id']
    seq_range   = args['range']
    get_rc      = args['rc']
    output_file = args['out']

    seq_range_split = seq_range.split('-')
    seq_range_l = int(seq_range_split[0])
    seq_range_r = seq_range_split[1]

    sequence_found = False
    output_file_handle = open(output_file, 'w')
    for seq_record in SeqIO.parse(seq_file, 'fasta'):
        if seq_record.id == ctg_id:

            sequence_found = True

            seq_subset = ''
            if seq_range_r == 'end':
                seq_subset = seq_record.seq[seq_range_l:]
            else:
                if int(seq_range_r) > len(seq_record.seq):
                    print('Specified end position longer than sequence, program exited!')
                    exit()
                else:
                    seq_subset = seq_record.seq[(seq_range_l - 1):int(seq_range_r)]

            if get_rc is True:
                seq_subset_rc = SeqRecord(seq_subset).reverse_complement().seq
                output_file_handle.write('>%s_%s_%s_rc\n' % (ctg_id, seq_range_l, seq_range_r))
                output_file_handle.write('%s\n' % seq_subset_rc)
            else:
                output_file_handle.write('>%s_%s_%s\n' % (ctg_id, seq_range_l, seq_range_r))
                output_file_handle.write('%s\n' % seq_subset)

    output_file_handle.close()

    if sequence_found is False:
        print('No sequence matched provided id, please double check!')


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()

    # arguments for select_seq
    parser.add_argument('-in',    required=True,                        help='sequence file')
    parser.add_argument('-id',    required=True,                        help='sequence id')
    parser.add_argument('-range', required=True,                        help='sequence range, start-end (in bp). e.g. 200-4000')
    parser.add_argument('-rc',    required=False, action='store_true',  help='write out reverse complement sequence')
    parser.add_argument('-out',   required=True,                        help='output file')

    args = vars(parser.parse_args())
    slice_seq(args)



import argparse
from Bio import SeqIO

parser = argparse.ArgumentParser()

parser.add_argument('-sam',
                    required=True,
                    help='Input sam file')

parser.add_argument('-ctgs',
                    required=True,
                    help='Contig list')

parser.add_argument('-seq',
                    required=True,
                    help='Reference sequences')

args = vars(parser.parse_args())
sam_file = args['sam']
ctgs_file = args['ctgs']
seq_file = args['seq']


# get contig list
ctg_list = []
for each_ctg in open(ctgs_file):
    each_ctg = each_ctg.strip()
    ctg_list.append(each_ctg)


for each_ref in ctg_list:

    # get sequence length
    ref_len = 0
    for each_seq in SeqIO.parse(seq_file, 'fasta'):
        if each_seq.id == each_ref:
            ref_len = len(each_seq.seq)
            #print('reference lenth %s\t%s' % (each_ref, ref_len))

    # get total length of mapped reads
    total_aligned_length = 0
    for each_align in open(sam_file):
        if each_align.startswith('@'):
            pass
        else:
            each_split = each_align.strip().split('\t')
            query_name = each_split[0]
            ref_name = each_split[2]
            query_seq_len = len(each_split[9])
            if ref_name == each_ref:
                #print('reads length %s\t%s' % (each_ref, query_seq_len))
                total_aligned_length += query_seq_len
    #print('total_aligned_length %s' % total_aligned_length)


    # calculate average coverage of current contig
    mean_coverage = float("{0:.2f}".format(total_aligned_length/ref_len))

    print('%s\t%s' % (each_ref, mean_coverage))



import os
import argparse
from Bio import SeqIO


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


# read in argument
parser = argparse.ArgumentParser()
parser.add_argument('-in', required=True, help='input fastq file')
args = vars(parser.parse_args())
fq_in = args['in']


# define output fasta file name
fq_in_path, fq_in_basename, fq_in_extension = sep_path_basename_ext(fq_in)
fa_out = '%s/%s.fa' % (fq_in_path, fq_in_basename)


# extract sequences
fa_out_handle = open(fa_out, 'w')
for seq_record in SeqIO.parse(fq_in, 'fastq'):
    seq_record_sequence = str(seq_record.seq)
    seq_record_description = seq_record.description
    fa_out_handle.write('>%s\n' % seq_record_description)
    fa_out_handle.write('%s\n' % seq_record_sequence)
fa_out_handle.close()



import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt


# https://seaborn.pydata.org/generated/seaborn.swarmplot.html#seaborn.swarmplot


# # load example dataset
# sns.set(style="whitegrid")
# tips = sns.load_dataset("tips")
# print(tips)
#
# # get the boxplot
# ax = sns.boxplot(x="sex", y="total_bill", data=tips)
#
# # Use swarmplot() to show the datapoints on top of the boxes
# ax = sns.swarmplot(x="sex", y="total_bill", data=tips, color='blue')

# ax = sns.stripplot(x="label", y="num", data=HGT_per_Mbp_list_with_class_df, color='orange', size=3, orient='v', jitter=True)



# # show the plot
# plt.show()
#
#

#
#
# d = {'col1': [1, 2], 'col2': [3, 4]}
# df = pd.DataFrame(data=d)
# print(df)
#
# df2 = pd.DataFrame(np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]]), columns=['a', 'b', 'c'])
# print(df2)
#
#
df3 = pd.DataFrame(np.array([[1, 1], [3, 1]]), columns=['a', 'b'])
print(df3)
#
#
sns.set(style="whitegrid")
#
# get the boxplot
ax = sns.boxplot(x="b", y="a", data=df3)

# Use swarmplot() to show the datapoints on top of the boxes
ax = sns.swarmplot(x="b", y="a", data=df3, color='blue')

plt.show()




from Bio import SeqIO
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC, generic_dna
from Bio import SeqFeature as SF
from Bio.SeqRecord import SeqRecord
from Bio.SeqFeature import SeqFeature, FeatureLocation


def export_dna_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.unambiguous_dna)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def export_aa_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.protein)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def get_genome_length(genome_file):

    total_length_bp = 0
    for each_seq in SeqIO.parse(genome_file, 'fasta'):
        total_length_bp += len(each_seq.seq)

    total_length_Mbp = float("{0:.2f}".format(total_length_bp / (1024 * 1024)))

    return total_length_Mbp


def sco2gbk():
    pass

def sco2ffn():
    pass

def sco2faa():
    pass

def gbk2ffn():
    pass


def gbk2faa():
    pass


def ffn2faa():
    pass




import argparse
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC
from Bio.SeqRecord import SeqRecord


# usage example
# python3 11_get_defined_region_sequence.py -f input.fasta -i contig_1 -s 5000 -e 10000 -o output.fasta
# python3 /Users/songweizhi/PycharmProjects/python_scripts/11_get_defined_region_sequence.py -f /Users/songweizhi/Desktop/FC/combined_ref.fasta -i 2.10_chromosome -s 866221 -e 941272 -o /Users/songweizhi/Desktop/2.10_chromosome_866221-941272.fasta


def export_dna_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.unambiguous_dna)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


parser = argparse.ArgumentParser(description='', add_help=False)
required = parser.add_argument_group('required arguments')
optional = parser.add_argument_group('optional arguments')
optional.add_argument('-h', action='help', help='Show this help message and exit')
required.add_argument('-f', dest='FILE', nargs='?', required=True,  type=str, help='Sequence file')
required.add_argument('-i', dest='SEQ_ID', nargs='?', required=True, type=str, help='Sequence ID')
required.add_argument('-s', dest='START_POS', nargs='?', required=True,  type=int, help='Start position')
required.add_argument('-e', dest='END_POS', nargs='?', required=True,  type=int, help='End position')
required.add_argument('-o', dest='OUTPUT', nargs='?', required=True,  type=str, help='End position')

args = vars(parser.parse_args())
ref_genome = args['FILE']
ref_seq = args['SEQ_ID']
start_pos = args['START_POS']
end_pos = args['END_POS']
output_file = args['OUTPUT']


output_file_handle = open(output_file, 'w')
for each_seq in SeqIO.parse(ref_genome, 'fasta'):
    if each_seq.id == ref_seq:
        seq_to_export = str(each_seq.seq[start_pos-1:end_pos])
        output_seq_id = '%s_%s_%s' % (ref_seq, start_pos, end_pos)
        export_dna_record(seq_to_export, output_seq_id, '', output_file_handle)
output_file_handle.close()




__author__ = 'weizhisong'


#################################################################################
#                                                                               #
#       This script was writen to format the NCBI blast results and put each    #
#                          orthology in one line                                #
#                                                                               #
#################################################################################

# The input file was generated by an all versus all blast step with the following parameter:
# /share/apps/blast+/2.2.31/bin/blastp -query -db -out -evalue 1e-5 -outfmt 6
blast_result = open('/Users/weizhisong/Desktop/HGT_temp/blast_results/all_vs_all_91bins_ffn.tab')
output = open('/Users/weizhisong/Desktop/HGT_temp/blast_results/orthology.txt', 'w')


gene_group = ''
subject_names = []

for each_line in blast_result:
        line_split = each_line.split("\t")
        query_name = line_split[0].strip('\n')
        subject_name = line_split[1].strip('\n')
        identity = float(line_split[2].strip('\n'))
        if query_name == gene_group and identity >= 50.00:
            subject_names.append(subject_name)
        else:
            if gene_group != '' and identity >= 50.00:
                output.write("%s\t%s" % (gene_group, '\t'.join(subject_names)) + '\n')
            gene_group = query_name
            subject_names = []
print gene_group

output.close()



import os
import argparse
from Bio import SeqIO

split_fasta_usage = '''
============ split_fasta example command ============

BioSAK split_fasta -i input.fa -o output_dir
BioSAK split_fasta -i input.fa -o output_dir -n 6

=====================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_ext = os.path.splitext(file_name)

    return file_path, file_basename, file_ext


def split_fasta(args):

    fasta_in         = args['i']
    per_file_seq_num = args['n']
    output_dir       = args['o']

    fasta_in_path, fasta_in_basename, fasta_in_ext = sep_path_basename_ext(fasta_in)

    if os.path.isdir(output_dir) is True:
        print('Output folder detected, please provide a different name!')
        exit()
    else:
        os.mkdir(output_dir)

    n = 1
    for seq_record in SeqIO.parse(fasta_in, 'fasta'):

        file_index = (n - 1) // per_file_seq_num + 1
        pwd_sub_file = '%s/%s_%s.fa' % (output_dir, fasta_in_basename, file_index)
        if per_file_seq_num == 1:
            pwd_sub_file = '%s/%s.fa' % (output_dir, seq_record.id)

        # write out sequence
        with open(pwd_sub_file, 'a') as pwd_sub_file_handle:
            pwd_sub_file_handle.write('>%s\n' % seq_record.id)
            pwd_sub_file_handle.write('%s\n'  % str(seq_record.seq))

        n += 1


if __name__ == '__main__':

    split_fasta_parser = argparse.ArgumentParser(usage=split_fasta_usage)
    split_fasta_parser.add_argument('-i', required=True,                       help='input fasta file')
    split_fasta_parser.add_argument('-n', required=False, type=int, default=1, help='sequence per file, default: 1')
    split_fasta_parser.add_argument('-o', required=True,                       help='output dir')
    args = vars(split_fasta_parser.parse_args())
    split_fasta(args)



import os
import dendropy
from dendropy import treecalc


os.chdir('/Users/songweizhi/Desktop')


tree = dendropy.Tree.get_from_path("species_tree.newick", "newick")
#pdm = treecalc.phylogenetic_distance_matrix(tree)
pdc = tree.phylogenetic_distance_matrix()
print(pdc)
print(type(pdc))



for i, t1 in enumerate(tree.taxon_namespace[:-1]):
    for t2 in tree.taxon_namespace[i+1:]:
        print("Distance between '%s' and '%s': %s" % (t1.label, t2.label, pdc(t1, t2)))
























import matplotlib.pyplot as plt
year = [1950, 1970, 1990, 2010]
pop = [2.5,3.7, 5.2, 7.0]
pop_2 = [1.5,2.7, 3.2, 6.0]
plt.plot(year,pop)
plt.scatter(year,pop)
plt.plot(year,pop_2)
plt.scatter(year,pop_2)
plt.show()




#!/usr/bin/python3
import argparse
from Bio import SeqIO


def export_seq(seq_record, single_line, output_handle):
    if single_line == 0:
        SeqIO.write(seq_record, output_handle, 'fasta')
    if single_line == 1:
        output_handle.write('>%s\n%s\n' % (seq_record.id, str(seq_record.seq)))


parser = argparse.ArgumentParser()

parser.add_argument('-in',
                    required=True,
                    help='input file')

parser.add_argument('-out',
                    required=True,
                    help='output file')

parser.add_argument('-step',
                    required=True,
                    type=int,
                    help='step')

parser.add_argument('-single_line',
                    action="store_true",
                    required=False,
                    help='to export sequences in single line')

args = vars(parser.parse_args())
input_file = args['in']
output_file = args['out']
step = args['step']
single_line = args['single_line']


n = 0
m = 0
output_handle = open(output_file, 'w')
for each_seq_record in SeqIO.parse(input_file, 'fasta'):
    n += 1
    if (n == 2*(step*m) + 1) or (n == 2*(step*m) + 2):
        export_seq(each_seq_record, single_line, output_handle)
        if n == 2*(step*m) + 2:
            m += 1
output_handle.close()



import os
import argparse


sampling_GTDB_gnms_usage = '''
====================================== sampling_GTDB_gnms example commands ======================================

# Example commands
BioSAK sampling_GTDB_gnms -p Firm_o -meta bac120_metadata_r202.tsv -taxon p__Firmicutes -r o
BioSAK sampling_GTDB_gnms -p Firm_f -meta bac120_metadata_r202.tsv -taxon p__Firmicutes -r f -cpl 85 -ctm 5
BioSAK sampling_GTDB_gnms -p Firm_g -meta bac120_metadata_r202.tsv -taxon p__Firmicutes -r g -cpl 85 -ctm 5 -ts

# -meta: bac120_metadata_r202.tsv or ar122_metadata_r202.tsv
# Sampling rank need to be lower than the rank of taxon specified with "-taxon".

=================================================================================================================
'''


def sampling_GTDB_gnms(args):

    output_prefix     = args['p']
    gtdb_gnm_metadata = args['meta']
    interested_taxon  = args['taxon']
    sampling_rank     = args['r']
    gnm_num_per_taxon = args['n']
    cpl_cutoff        = args['cpl']
    ctm_cutoff        = args['ctm']
    type_strain_only  = args['ts']

    # define output file name
    selected_gnm_id_txt   = '%s_selected_gnm_id.txt'       % output_prefix
    selected_gnm_meta_txt = '%s_selected_gnm_metadata.txt' % output_prefix

    to_exit = False
    if os.path.isfile(selected_gnm_id_txt) is True:
        print('%s detected, please use a different prefix' % selected_gnm_id_txt)
        to_exit = True

    if os.path.isfile(selected_gnm_meta_txt) is True:
        print('%s detected, please use a different prefix' % selected_gnm_meta_txt)
        to_exit = True

    if to_exit is True:
        print('Program exited!')
        exit()

    dod = {}
    gnm_cpl_dict = {}
    gnm_ctm_dict = {}
    gnm_size_dict = {}
    gnm_metric_dict = {}
    gnm_to_taxon_dict = {}
    col_index = {}
    for each_ref in open(gtdb_gnm_metadata):
        each_ref_split = each_ref.strip().split('\t')
        if each_ref.startswith('accession	ambiguous_bases'):
            col_index = {key: i for i, key in enumerate(each_ref_split)}
        else:
            ref_accession = each_ref_split[0][3:]
            gnm_completeness = float(each_ref_split[2])
            gnm_contamination = float(each_ref_split[3])
            gnm_size = each_ref_split[col_index['genome_size']]
            gtdb_taxonomy = each_ref_split[col_index['gtdb_taxonomy']]
            is_type = each_ref_split[col_index['gtdb_type_designation']]

            if interested_taxon in gtdb_taxonomy:

                # check type
                type_pass = False
                if type_strain_only is False:
                    type_pass = True
                else:
                    if is_type in ['type strain of species', 'type strain of species']:
                        type_pass = True

                # check completeness
                cpl_passed = False
                if cpl_cutoff is None:
                    cpl_passed = True
                else:
                    if gnm_completeness >= float(cpl_cutoff):
                        cpl_passed = True

                # check contamination
                ctm_passed = False
                if ctm_cutoff is None:
                    ctm_passed = True
                else:
                    if gnm_contamination <= float(ctm_cutoff):
                        ctm_passed = True

                if (type_pass is True) and (cpl_passed is True) and (ctm_passed is True):

                    # add to dict
                    gnm_to_taxon_dict[ref_accession] = gtdb_taxonomy
                    gnm_cpl_dict[ref_accession] = gnm_completeness
                    gnm_ctm_dict[ref_accession] = gnm_contamination
                    gnm_size_dict[ref_accession] = gnm_size
                    gnm_quality_metric = gnm_completeness - (5 * gnm_contamination)
                    gnm_metric_dict[ref_accession] = float("{0:.3f}".format(gnm_quality_metric))

                    # get taxon at sampling rank
                    sampling_rank_taxon = ''
                    for each_rank_taxon in gtdb_taxonomy.split(';'):
                        if each_rank_taxon[:3] == '%s__' % sampling_rank:
                            sampling_rank_taxon = each_rank_taxon

                    if sampling_rank_taxon not in dod:
                        dod[sampling_rank_taxon] = dict()

                    dod[sampling_rank_taxon][ref_accession] = gnm_quality_metric

    selected_gnm_id_txt_handle = open(selected_gnm_id_txt, 'w')
    selected_gnm_meta_txt_handle = open(selected_gnm_meta_txt, 'w')
    selected_gnm_meta_txt_handle.write('Rank\tGenome\tCompleteness\tContamination\tSelection_criterion(completeness-5*contamination)\tSize(bp)\tTaxonomy\n')
    selected_gnm_list = []
    for each_sampling_taxon in dod:
        current_taxon_gnm_quality_dict = dod[each_sampling_taxon]
        gnms_sorted_by_quality = sorted(current_taxon_gnm_quality_dict.items(), key=lambda x: x[1], reverse=True)
        selected_gnm = 0
        for each_gnm in gnms_sorted_by_quality:
            if selected_gnm < gnm_num_per_taxon:
                gnm_id = each_gnm[0]
                selected_gnm_list.append(gnm_id)
                str_to_write = '%s\t%s\t%s\t%s\t%s\t%s\t%s' % (each_sampling_taxon, gnm_id,
                                                               gnm_cpl_dict[gnm_id],
                                                               gnm_ctm_dict[gnm_id],
                                                               gnm_metric_dict[gnm_id],
                                                               gnm_size_dict[gnm_id],
                                                               gnm_to_taxon_dict[gnm_id])
                selected_gnm_meta_txt_handle.write(str_to_write + '\n')
                selected_gnm_id_txt_handle.write(gnm_id + '\n')
                selected_gnm += 1
    selected_gnm_id_txt_handle.close()
    selected_gnm_meta_txt_handle.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-p',           required=False, default='Genomes',          help='output prefix')
    parser.add_argument('-meta',        required=True,                              help='GTDB reference genome metadata')
    parser.add_argument('-taxon',       required=True,                              help='interested taxon')
    parser.add_argument('-r',           required=True,                              help='sampling rank, select from p, c, o, f, g and s')
    parser.add_argument('-n',           required=False, default=1, type=int,        help='numer of genome to retain per taxon')
    parser.add_argument('-cpl',         required=False, default=None, type=float,   help='completeness cutoff (0-100), default: None')
    parser.add_argument('-ctm',         required=False, default=None, type=float,   help='contamination cutoff, default: None')
    parser.add_argument('-ts',          required=False, action='store_true',        help='only consider type strain')
    args = vars(parser.parse_args())
    sampling_GTDB_gnms(args)



import os
import glob
import shutil
import argparse
import configparser

usage = """

    Usage:

    python3 /srv/scratch/z5039045/Softwares/MetaCHIP/InPrep1_Prokka.py -genome_folder Kamchatka_Bins_qualified_40_0 -x fasta -o prokka_output

    It will:
    1. Generate qsub file to run Prokka for each input genome
    2. Submit generated qsub files

"""

####################################################### Configuration ##################################################

parser = argparse.ArgumentParser()

parser.add_argument('-f',
                    required=True,
                    help='folder name of input genomes')

parser.add_argument('-x',
                    required=True,
                    help='genome_file_extension')

parser.add_argument('-o',
                    required=True,
                    help='output folder')

args = vars(parser.parse_args())
genome_folder = args['f']
genome_extension = args['x']
output_folder = args['o']

prokka_modules = ['perl/5.20.1', 'infernal/1.1.1', 'blast+/2.6.0', 'hmmer/3.1b2', 'prodigal/2.6.3', 'tbl2asn/25.6', 'parallel/20160222', 'prokka/1.12']

########################################################################################################################


# get working directory
working_directory = os.getcwd()

# get genome name list
genome_list = [os.path.basename(file_name) for file_name in glob.glob('%s/%s/*.%s' % (working_directory, genome_folder, genome_extension))]
print('%i genomes were found in total.' % len(genome_list))

if os.path.isdir(output_folder):
    shutil.rmtree(output_folder)
    if os.path.isdir(output_folder):
        shutil.rmtree(output_folder)
    os.mkdir(output_folder)
else:
    os.mkdir(output_folder)

# prepare prokka qsub file header
line_1 = '#!/bin/bash\n'
line_2 = '#PBS -l nodes=' + str(1) + ':ppn=' + str(1) + '\n'
line_3 = '#PBS -l vmem=' + str(10) + 'gb\n'
line_4 = '#PBS -l walltime=' +'00:59:00' + '\n'
line_5 = '#PBS -j oe\n'
line_6 = '#PBS -M ' + 'wythe1987@163.com' + '\n'
line_7 = '#PBS -m ae'
header = line_1 + line_2 + line_3 + line_4 + line_5 + line_6 + line_7

if os.path.isdir('qsub_files'):
    pass
else:
    os.mkdir('./qsub_files')

for genome in genome_list:
    genome_name = genome.split('.')[0]
    prokka_qsub_out = open('%s/qsub_files/qsub_prokka_%s.sh' % (working_directory, genome_name), 'w')
    # check whether folder exist
    if os.path.isdir('./%s/%s' % (output_folder, genome_name)):
        shutil.rmtree('./%s/%s' % (output_folder, genome_name))
        os.mkdir('./%s/%s' % (output_folder, genome_name))
    else:
        os.mkdir('./%s/%s' % (output_folder, genome_name))

    prokka_qsub_out.write(header + '\n\n')
    for module in prokka_modules:
        prokka_qsub_out.write('module load %s\n' % module)
    prokka_qsub_out.write('prokka --force --metagenome --cpus 1 --compliant --prefix %s --locustag %s --strain %s --outdir %s/%s/%s %s/%s/%s\n'
                          % (genome_name, genome_name, genome_name, working_directory, output_folder, genome_name, working_directory, genome_folder, genome))
    prokka_qsub_out.close()

# submit qsub files
qsub_file_list = [os.path.basename(file_name) for file_name in glob.glob('./qsub_files/*.sh')]
current_wd = os.getcwd()
os.chdir('./qsub_files')
for qsub_file in qsub_file_list:
    os.system('qsub %s' % qsub_file)
os.chdir(current_wd)



import os
import sys
from time import sleep
from Bio import SeqIO

usage = """
#####################################################################################

    remove_unpaired_reads.py - remove unpaired reads from 2 paired raw reads files.

    Usage:
    python3 remove_unpaired_reads.py [prefix_R1.fastq] [prefix_R2.fastq]

    Reminder: It needs about 3 hours to process 30GB input reads (two reads files
    in total, uncompressed).

#####################################################################################
"""

if len(sys.argv) != 3:
    print(usage)
    exit(1)

reads_1_file = sys.argv[1]
reads_2_file = sys.argv[2]


def get_fastq_reads_id_list(reads_file, file_extension):
    reads = SeqIO.parse(reads_file, file_extension)
    reads_id_list = []
    total_reads = 0
    for each_read in reads:
        reads_id_list.append(each_read.id)
        total_reads += 1
    return reads_id_list, total_reads


def get_new_reads_file(input_reads_file, common_reads_list, output_reads_file, file_extension):
    out = open(output_reads_file, 'w')
    input_reads = SeqIO.parse(input_reads_file, file_extension)
    for each_read in input_reads:
        if each_read.id in common_reads_list:
            SeqIO.write(each_read, out, file_extension)
    out.close()


# check file extension
path_file_1, file_1_name = os.path.split(reads_1_file)
path_file_2, file_2_name = os.path.split(reads_2_file)
file_1_name_split = file_1_name.split('.')
file_2_name_split = file_2_name.split('.')
file_1_only_name = file_1_name_split[0]
file_2_only_name = file_2_name_split[0]
file_1_extension = file_1_name_split[-1]
file_2_extension = file_2_name_split[-1]
if file_1_extension != file_2_extension:
    print('Please use same file extension for the 2 input reads files')
    exit()

# get output file name
if path_file_1 == '':
    path_file_1 = '.'
if path_file_2 == '':
    path_file_2 = '.'
file_extension = file_1_extension

file_1_out_name = '%s_only_paired.%s' % (file_1_only_name, file_extension)
file_2_out_name = '%s_only_paired.%s' % (file_2_only_name, file_extension)
pwd_file_1_out = '%s/%s' % (path_file_1, file_1_out_name)
pwd_file_2_out = '%s/%s' % (path_file_2, file_2_out_name)

# get first reads id list
print('Get reads id list from %s' % file_1_name)
sleep(0.5)
file1_reads_id_list, file1_total_reads = get_fastq_reads_id_list(reads_1_file, file_extension)
print('%s reads found in %s' % (file1_total_reads, file_1_name))
sleep(0.5)

# get second reads id list
print('Get reads id list from %s' % file_2_name)
sleep(0.5)
file2_reads_id_list, file2_total_reads = get_fastq_reads_id_list(reads_2_file, file_extension)
print('%s reads found in %s' % (file2_total_reads, file_2_name))
sleep(0.5)

# get common reads id list
print('Get paired reads list...')
sleep(0.5)
common_reads_list = set(file1_reads_id_list).intersection(file2_reads_id_list)
total_common_reads = len(common_reads_list)
print('%s paired reads found between %s and %s' % (total_common_reads, file_1_name, file_2_name))
sleep(0.5)

if (file1_total_reads == total_common_reads) and (file2_total_reads == total_common_reads):
    print('All reads in %s and %s are paired, program exited' % (file_1_name, file_2_name))
    exit()

# get output files
print('Remove unpaired reads from %s' % file_1_name)
sleep(0.5)
get_new_reads_file(reads_1_file, common_reads_list, pwd_file_1_out, file_extension)
print('Unpaired reads removed and saved paired reads to %s' % pwd_file_1_out)
sleep(0.5)
print('Remove unpaired reads from %s' % file_2_name)
sleep(0.5)
get_new_reads_file(reads_2_file, common_reads_list, pwd_file_2_out, file_extension)
print('Unpaired reads removed and saved paired reads to %s' % pwd_file_2_out)
sleep(0.5)
print('Done!')



import requests
import json
from lxml import etree
import time
import datetime


#获取网页相应内容
def get_one_page(url):
    try:
        headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/65.0.3325.181 Safari/537.36'}
        response = requests.get(url, headers=headers)
        if response.status_code == 200:
            return response.text
        return None
    except:
        print("访问出错")
        return None


def get_one_page_pdf(url):
    try:
        headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/65.0.3325.181 Safari/537.36'}
        response = requests.get(url, headers=headers)
        print(response.status_code)
        if response.status_code == 200:
            return response
        return None
    except:
        print("访问出错")
        return None


def parse_one_page(html):
    html = etree.HTML(html)
    title = html.xpath('//*[@id="main-content"]/main/article/div/h1/text()')
    Accesses = html.xpath('//*[@id="main-content"]/main/article/div/div/ul/li[1]/p//text()')
    DOI = html.xpath('//meta[@name="prism.doi"]/@content')
    published = html.xpath('//*[@id="main-content"]/main/article/div/ul[1]/li[3]/a/time/text()')
    Citations = html.xpath('//*[@id="main-content"]/main/article/div/div/ul/li[2]/p/text()')
    Altmetric = html.xpath('//*[@id="main-content"]/main/article/div/div/ul/li[3]/p/text()')
    author = html.xpath('//*[@id="main-content"]/main/article/div/ul[2]//text()')
    list = []
    for i in range(len(author)):
        if len(author[i]) > 6:
            list.append(author[i])
    author= list[:-3]
    print()
    yield {
        'Title': title,
        'Accesses': Accesses,
        'DOI': DOI,
        'Citations':Citations,
        'Altmetric':Altmetric,
        'published':published,
        'author':author
    }


def get_paper_url(n):

    #查找关键字网址
    url = 'https://microbiomejournal.biomedcentral.com/articles?searchType=journalSearch&sort=PubDate&page={num}'.format(num=n)

    # 获取相应内容
    html = get_one_page(url)

    # xpath解析文献href
    html = etree.HTML(html)
    href = html.xpath('//h3/a[@data-test="title-link"]/@href')

    #对每个href进行关键字筛选
    for i in href:
        #单篇文献url
        url = 'https://microbiomejournal.biomedcentral.com'+str(i)
        doi = i.replace('/articles','')
        url_pdf = 'https://microbiomejournal.biomedcentral.com/track/pdf'+doi+'#page=1'
        print(url_pdf)
        html = get_one_page(url)
        name = []
        #判断关键字是否存在于文献中
        for item in parse_one_page(html):
                with open('data.csv', 'a', encoding='utf-8') as f:
                    f.write(json.dumps(item['Title'], ensure_ascii=False)+ ',')
                    f.write(json.dumps(item['DOI'], ensure_ascii=False)+ ',')
                    f.write(json.dumps(item['Accesses'], ensure_ascii=False)+ ',')
                    f.write(json.dumps(item['Citations'], ensure_ascii=False)+ ',')
                    f.write(json.dumps(item['Altmetric'], ensure_ascii=False) + ',')
                    f.write(json.dumps(item['published'], ensure_ascii=False) + ',')
                    f.write(json.dumps(item['author'], ensure_ascii=False)+ '\n')
                    time.sleep(2)
                    f.close()
                print('======读取pdf中=====')
                r = get_one_page_pdf(url_pdf)
                print('======保存pdf中=====')
                with open('data_pdf/'+item['Title'][0]+'.pdf', 'wb') as p:
                    p.write(r.content)
                    print('======保存完毕=====')
                    p.close()
                time.sleep(2)


def main():

    for i in range(1, 18):
        n = i
        try:
            starttime = datetime.datetime.now()
            get_paper_url(n)
            endtime = datetime.datetime.now()
            duringtime = endtime - starttime
            print('第%d页完成' %(n))
            print(duringtime.seconds)
        except:
            print("出错")
            print("第%d页出错" % (n))
            continue


if __name__ == '__main__':
    main()



import os
import glob
import pandas
import argparse
import numpy as np


'''

Python3 ~/PycharmProjects/BioSAK/BioSAK/boxplot_matrix_COG.py -in Kelp_dRep_fun_stats -out Kelp_dRep_fun_stats.txt -in_percent 
Python3 ~/PycharmProjects/BioSAK/BioSAK/boxplot_matrix_COG.py -in Tara_NM_fun_stats -out Tara_NM_fun_stats.txt -in_percent

Rscript ~/PycharmProjects/BioSAK/BioSAK/COG_boxplot_last1row.R -i Kelp_dRep_fun_stats.txt -o Kelp_dRep_fun_stats.png
Rscript ~/PycharmProjects/BioSAK/BioSAK/COG_boxplot_last1row.R -i Tara_NM_fun_stats.txt -o Tara_NM_fun_stats.png

cd /Users/songweizhi/Desktop/validate_MetaCHIP
Python3 ~/PycharmProjects/BioSAK/BioSAK/boxplot_matrix_COG.py -in dRep95_fun_stat_files_sponge -out dRep95_fun_stat_sponge.txt -in_percent -skip_1st_row
Python3 ~/PycharmProjects/BioSAK/BioSAK/boxplot_matrix_COG.py -in dRep99_fun_stat_files_sponge -out dRep99_fun_stat_sponge.txt -in_percent -skip_1st_row

Rscript ~/PycharmProjects/BioSAK/BioSAK/COG_boxplot_last1row.R -i dRep95_fun_stat_sponge.txt -o dRep95_fun_stat_sponge.png
Rscript ~/PycharmProjects/BioSAK/BioSAK/COG_boxplot_last1row.R -i dRep99_fun_stat_sponge.txt -o dRep99_fun_stat_sponge.png

'''


def turn_to_percentage(number_list):
    number_list_percent = []
    for each_element in number_list:
        each_element_percent = float("{0:.2f}".format(each_element / sum(number_list)))
        number_list_percent.append(each_element_percent)
    return number_list_percent


parser = argparse.ArgumentParser()

parser.add_argument('-in',                          required=True,                          help='folder holds the func_stats.txt file for each genome')
parser.add_argument('-out',                         required=True,                          help='output csv file')
parser.add_argument('-columns',                     required=False,                         help='the order of columns, e.g. "J,A,K,L,B,D,Y,V,T,M,N,Z,W,U,O,X,C,G,E,F,H,I,P,Q,R,S" for COG categories')
parser.add_argument('-in_percent',                  required=False, action="store_true",    help='in percent')
parser.add_argument('-skip_1st_row',                required=False, action="store_true",    help='skip 1st (header) row')
parser.add_argument('-with_functional_description', required=False, action="store_true",    help='add functional description to column header')

args = vars(parser.parse_args())
input_folder =                  args['in']
output_csv =                    args['out']
column_order =                  args['columns']
in_percent =                    args['in_percent']
skip_1st_row =                  args['skip_1st_row']
with_functional_description =   args['with_functional_description']

column_order_list = ['J', 'A', 'K', 'L', 'B', 'D', 'Y', 'V', 'T', 'M', 'N', 'Z', 'W', 'U', 'O', 'X', 'C', 'G', 'E', 'F', 'H', 'I', 'P', 'Q', 'R', 'S']

input_files = '%s/*.txt' % input_folder
file_list = [os.path.basename(file_name) for file_name in glob.glob(input_files)]
file_list = sorted(file_list)


############################################ get category_num_lol with dict ############################################

genome_name_list = []
detected_category_id_list = set()
annotation_results_dict = {}
cate_to_description_dict = {}
for each_file in file_list:
    genome_name = '_'.join(each_file.split('_')[:-2])
    genome_name_list.append(genome_name)

    current_annotation_results_dict = {}
    n = 0
    for each_category in open('%s/%s' % (input_folder, each_file)):
        each_category_split = each_category.strip().split('\t')
        category_id =           each_category_split[0]
        category_num_str =      each_category_split[1]
        category_description =  each_category_split[2]

        if skip_1st_row is True:
            if n > 0:
                current_annotation_results_dict[category_id] = int(category_num_str)
                detected_category_id_list.add(category_id)
        else:
            current_annotation_results_dict[category_id] = int(category_num_str)
            detected_category_id_list.add(category_id)

        if category_id not in cate_to_description_dict:
            cate_to_description_dict[category_id] = category_description

        n += 1

    annotation_results_dict[genome_name] = current_annotation_results_dict


# reorder columns

column_order_list_detected = []
for cate_id in column_order_list:
    if cate_id in detected_category_id_list:
        column_order_list_detected.append(cate_id)


column_order_list_detected_with_description = []
for detected_cate in column_order_list_detected:
    column_order_list_detected_with_description.append('%s_%s' % (detected_cate, cate_to_description_dict[detected_cate]))

# get category_num_lol
category_num_lol = []
for genome in genome_name_list:

    current_genome_annotation_results_dict = annotation_results_dict[genome]
    current_genome_annotation_results_list = []
    for cog_cate in column_order_list_detected:
        cog_cate_num = 0
        if cog_cate in current_genome_annotation_results_dict:
            cog_cate_num = current_genome_annotation_results_dict[cog_cate]
        current_genome_annotation_results_list.append(cog_cate_num)

    # turn absolute number to percentage if specified
    if in_percent is True:
        current_genome_annotation_results_list_in_percent = turn_to_percentage(current_genome_annotation_results_list)
        category_num_lol.append(current_genome_annotation_results_list_in_percent)
    else:
        category_num_lol.append(current_genome_annotation_results_list)


########################################################################################################################

# turn list into arrary
category_num_arrary = np.array(category_num_lol)

# add row and column name to dataframe
if with_functional_description is True:
    category_num_df = pandas.DataFrame(category_num_arrary, index=genome_name_list, columns=column_order_list_detected_with_description)
else:
    category_num_df = pandas.DataFrame(category_num_arrary, index=genome_name_list, columns=column_order_list_detected)

# write out
category_num_df.to_csv(output_csv)
